<?php
/**
 * Executa manipulacao de valores.
 *
 * @package    Geral
 * @subpackage Logic
 * @author     alessandrobartels 31/08/2011
 */
class Valor {

    const TOTAL = 'TOTAL';
    const PARCIAL = 'PARCIAL';
    const TRUNCA  = 'TRUNCA';
    const STR_ID_DECIMAL = '.';
    const STR_ZERO = '0';
    const STR_UM = '1';
    const STR_VAZIO = '';
    const NR_CINCO = 5;
    const NR_DEZ = 10;


    /**
     * Converte um valor em dinheiro.
     *
     * @param float  $valor             Valor a ser arredondado.
     * @param string $tipoArredonda     Tipo de arredondamento.
     * @param float  $nrDigitoArredonda Quantidade de casas decimais para o arredondamento.
     * @param float  $vrLimiteArredonda Digito que vai gerar o arredondamento.
     *
     * @return float
     */
    public static function arredonda($valor, $tipoArredonda, $nrDigitoArredonda, $vrLimiteArredonda) {

        // Converte dinheiro.
        if ($tipoArredonda === self::TOTAL && (int)$nrDigitoArredonda === -2 && (int)$vrLimiteArredonda === 5) {
            $aux = round($valor, abs($nrDigitoArredonda));
            $dif = $aux - $valor;
            if ($dif > 0) {
                /* Quando o valor chega arredondado a chamada do round novamente gera uma imprecisao acima da casa
                 * de milesimos. Como o intuito eh comparar dinheiro, se a imprecisao estiver acima da quarta casa
                 * retorna o mesmo valor parametrizado.
                 */
                if ($dif < 0.0001) {
                    return $valor;
                } else {
                    return $aux;
                }
            } else {
                return $aux;
            }
        }

        // INSS
        if ($tipoArredonda === self::TRUNCA && (int)$nrDigitoArredonda === -2) {

                $valor = floor(round($valor, 2) * 1000)/1000;
                return $valor;
            
        }

        if ($tipoArredonda === self::TRUNCA && (int)$nrDigitoArredonda === 0 && (int)$vrLimiteArredonda === 0) {
            $valor = ceil($valor);
            return $valor;
        }

        // Metodo generico para o arredondamento.
        $aux = self::_trataTipoArredondamento($valor, $tipoArredonda, $nrDigitoArredonda, $vrLimiteArredonda);
        return $aux;

    }


    public static function atribuiCasaDecimal($valor, $nrDigitoDecimal = 2 ){
        if (is_string($valor) === false) {
            $valor = number_format($valor, $nrDigitoDecimal, self::STR_ID_DECIMAL, self::STR_VAZIO);
        }

        return $valor;
    }


    /**
     * Trata o tipo de arredondamento conforme o tipo.
     *
     * @param float   $valor Valor a ser manipulado.
     * @param string  $tipoArredonda Tipo de arredondamento.
     * @param integer $nrDigitoArredonda Quantidade de casas decimais para o arredondamento.
     * @param integer $vrLimiteArredonda Digito que vai gerar o arredondamento.
     *
     * @return float
     *
     * @author alessandrobartels - 15/12/2011.
     */
    private static function _trataTipoArredondamento($valor, $tipoArredonda, $nrDigitoArredonda, $vrLimiteArredonda){

        switch ($tipoArredonda) {
        case 'TRUNCA':
            return self::_arredondaTrunca($valor, $nrDigitoArredonda);
            break;
        case 'TOTAL':
            return self::_arredondaTotal($valor, $nrDigitoArredonda, $vrLimiteArredonda);
            break;
        case 'PARCIAL':
            return self::_arredondaParcial($valor, $nrDigitoArredonda, $vrLimiteArredonda);
            break;
        default:
            return $valor;
            break;
        }
    }


    /**
     * Trunca um valor.
     *
     * @param float   $valor             Valor a ser manipulado.
     * @param integer $nrDigitoArredonda Quantidade de casas decimais para o arredondamento.
     *
     * @return float
     *
     * @example Valor: 253,467
     * 			Digito: -2  ==> 253,46
     * 			Digito:  0  ==> 253
     * 			Digito:  1  ==> 250
     *
     * @author alessandrobartels - 14/12/2011
     */
    private static function _arredondaTrunca($valor, $nrDigitoArredonda){

        $dig = abs($nrDigitoArredonda);
        $pos = strpos($valor, self::STR_ID_DECIMAL);

        if ($nrDigitoArredonda < 0) {
            /* Depois da virgula (parte decimal). */
            $aux = substr($valor, 0, ($pos+$dig+1));
        } else {
            /* Antes da virgula (parte inteira). */

            // Valida o ponto de inicio da substituicao para nao ser negativo. Se fosse, a substituicao seria iniciada
            // do final da string.
            $start = $pos-$dig;
            if ($start < 0) {
                $start = 0;
                $dig = $pos;
            }

            // Preenche a string de substituicao.
            $sub = str_pad(self::STR_VAZIO, $dig, self::STR_ZERO);

            // Substitui a sub-string acima na posicao demarcada.
            $aux = substr_replace($valor, $sub, $start, $dig);

            // Retira a sub-string antes da virgula.
            $aux = substr($aux, 0, $pos);
        }

        return (float)$aux;
    }


    /**
     * Arredonda Valor (multiplos de 10).
     *
     * @param float   $valor             Valor a ser manipulado.
     * @param integer $nrDigitoArredonda Quantidade de casas decimais para o arredondamento.
     * @param integer $vrLimiteArredonda Digito que vai gerar o arredondamento.
     *
     * @return float
     *
     * @example Valor: 253,467
     * 			Limite: 5 Digito: -2  ==> 253,47
     * 			Limite: 2 Digito: -1  ==> 253,5
     * 			Limite: 5 Digito:  0  ==> 253
     * 			Limite: 3 Digito:  0  ==> 254
     * 			Limite: 5 Digito:  1  ==> 253
     * 			Limite: 1 Digito:  1  ==> 260
     *
     * @author alessandrobartels - 14/12/2011
     */
    private static function _arredondaTotal($valor, $nrDigitoArredonda, $vrLimiteArredonda){
        // CASTING FORÇADO PARA SATISFAZER A CONDIÇÃO IF === QUE TESTA $nrDigitoArredonda   BORBA 27/02/2013
//tk_warning('valor '.$valor);

        $decimal = $valor - round($valor, 0, PHP_ROUND_HALF_DOWN);
        //Matheus Silva - 23/02/2015
        if($decimal<0.01){
            return $valor;
        }

        //BORBA - 18/07/2013 - Criado um auxiliar "casteado"
        (string)$strValor=$valor;
//tk_warning('str fora '.$strValor);
        $nrDigitoArredonda=(int)$nrDigitoArredonda;
        $dig = abs($nrDigitoArredonda);
        $pos = strpos($strValor, self::STR_ID_DECIMAL);

        /*BORBA - 09/12/2014 para 9/99/999/9999/99999 etc.. pos deve deixar de decrementar 1 Atendendo à solicitação */
        $valorCabal=substr($valor, 0, $pos);

        //BORBA - 18/07/2013 Para valores que já chegam redondos pos recebe nulo, concatena-se um decimal em
        //                   $strValor e recalcula-se $pos
        if ($pos == null){
            $strValor=$strValor.'.000000000';
//tk_warning('str dentro '.$strValor);
            $pos = strpos($strValor, self::STR_ID_DECIMAL);
        }
        $lim = abs($vrLimiteArredonda);
//tk_warning('pos '.$pos);
        if ($nrDigitoArredonda <= 0) {
            /* ########### Depois da virgula (casas decimais) ########### */

            // Acrescenta uma posicao no digito para desprezar a posicao da virgula.
            $dig++;

            // Valor da posicao do digito apos a virgula.
            $limiteAux = substr($strValor, ($pos+$dig), 1);
            $limiteAux1 = substr($strValor, ($pos+$dig), 10);

            (int)$limiteAux2=$limiteAux1;

            //$limiteAux = substr($valor, ($pos+$dig), 1);
//tk_warning('limite '.$limiteAux);

            //BORBA - 18/07/2013 - Cast feito em $limiteAux antes de ser comparado a $lim que é int
            ///Acrescentado ||($lim==1 && (int)$limiteAux2>0)) para tratar a excessao digito 0 limite 1
            if (((int)$limiteAux >= $lim )||($lim==1 && (int)$limiteAux2>0)) {
                /* Maior ou igual ao limite. */

                // Monta sub-string para somar com o valor a ser arredondado - DECIMAL (Ex. 1, 0.1, 0.01, 0.001 ...).
                if ($nrDigitoArredonda === 0) {
//tk_warning('entrei if interno');
                    $sub = 1;
                } else {
                    $sub = self::STR_ZERO.self::STR_ID_DECIMAL;
                    $sub = str_pad($sub, $dig, self::STR_ZERO);
                    $sub = str_pad($sub, (strlen($sub)+1), self::STR_UM);
                }

                // Executa arredondamento -- Soma valor mais DECIMAL a arredondar.
                $valor = $valor + (float)$sub;
//tk_warning('->  '.$valor);
            }

            /*BORBA - 09/12/2014 para 9/99/999/9999/99999 etc.. pos deve incrementar 1 pois 9(1posição) vira para 10(2posições) e assim sucessivamente Atendendo à solicitação */
            $listaCabal = array('9','99','999','9999','99999');
            if(in_array($valorCabal,$listaCabal)){
                $pos++;
            }

            /* Validacao para nao imprimir a virgula quando o arredondamento eh no primeiro decimal*/
            if ($nrDigitoArredonda === 0) {
//tk_warning('entrei if externoo');
                $dig--;
            }

            $aux = substr($valor, 0, ($pos+$dig));
//tk_warning('aux '.$aux);
        } else {
            /* ########### Antes da virgula (parte inteira) ########### */
            $start = $pos-$dig;

            if ($start < 0) {
                $start = 0;
                $dig = $pos;
            }

            $limiteAux = substr($valor, ($pos-$dig), 1);

            if ($limiteAux >= $lim) {
                /* Maior ou igual ao limite. */

                // Monta sub-string para somar com o valor a ser arredondado - INTEIRO (Ex. 1, 10, 1000 ...).
                $sub = self::STR_UM;
                $sub = str_pad($sub, ($dig+1), self::STR_ZERO);
                // Executa arredondamento -- Soma valor mais INTEIRO a arredondar.
                $aux = $valor + (float)$sub;

                // Monta sub-string com zero (Ex. 0, 00, 000).
                $sub = str_pad(self::STR_VAZIO, $dig, self::STR_ZERO);

                // Tratamento para arredondamento no primeiro algarismo.
                if ($start === 0) {
                    $start++;
                    $pos++;
                }

                // Substitui a sub-string acima na posicao demarcada.
                $valor = substr_replace($aux, $sub, $start, $dig);
            }

            $aux = substr($valor, 0, $pos);
        }

        return (float)$aux;
    }


    /**
     * Arredonda Valor (multiplos de 5).
     *
     * @param float   $valor             Valor a ser manipulado.
     * @param integer $nrDigitoArredonda Quantidade de casas decimais para o arredondamento.
     * @param integer $vrLimiteArredonda Digito que vai gerar o arredondamento.
     *
     * @return float
     *
     * @example Valor: 253.434398
     * 			Limite: 5 Digito: -2  ==> 253,435
     * 			Limite: 2 Digito: -1  ==> 253,5
     * 			Limite: 1 Digito:  0  ==> 254
     * 			Limite: 5 Digito:  1  ==> 255
     * 			Limite: 1 Digito:  2  ==> 300
     * 			Limite: 1 Digito:  3  ==> 1000
     *
     * @author alessandrobartels - 15/12/2011
     */
    private static function _arredondaParcial($valor, $nrDigitoArredonda, $vrLimiteArredonda){
        $dig = abs($nrDigitoArredonda);
        $pos = strpos($valor, self::STR_ID_DECIMAL);
        $lim = abs($vrLimiteArredonda);

        if ($nrDigitoArredonda <= 0) {
            /* ########### Depois da virgula (casas decimais) ########### */

            // Acrescenta uma posicao no valor absoluto do digito para desprezar a posicao da virgula.
            $dig++;

            // Decrementa uma posicao no valor do digito para desprezar a posicao da virgula.
            $nrDigitoArredonda--;

            // Valor da posicao do digito apos a virgula.
            $limiteAux = substr($valor, ($pos+$dig), 1);

            // Calcula o valor que sera acrescido ao digito que sera arredondado conforme o limite.
            if ($limiteAux >= $lim) {
                $posAux = 0;
                $sub = self::_executaArredondamento($valor, $pos, $nrDigitoArredonda, self::NR_DEZ);
            } else {
                $posAux = 1;
                $sub = self::_executaArredondamento($valor, $pos, $nrDigitoArredonda, self::NR_CINCO);
            }

            // Arredonda.
            $valor = $valor + (float)$sub;

            /* Validacao para nao imprimir a virgula quando o arredondamento eh no primeiro decimal*/
            if ($nrDigitoArredonda === 0) {
                $dig--;
            }

            /* $posAux ===> variavel auxiliar para extrair o valor requerido do valor calculado. */
            $aux = substr($valor, 0, ($pos+$dig+$posAux));
        } else {
            /* ########### Antes da virgula (parte inteira) ########### */
            $start = $pos-$dig;

            // Valida para que o primiero digito seje o mais a esquerda da virgula.
            if ($start < 0) {
                $start = 0;
                $dig = $pos;
            }

            // Calcula o valor que sera acrescido ao digito que sera arredondado conforme o limite.
            $limiteAux = substr($valor, ($pos-$dig), 1);
            if ($limiteAux >= $lim) {
                $sub = self::_executaArredondamento($valor, $pos, $nrDigitoArredonda, self::NR_DEZ);
            } else {
                $sub = self::_executaArredondamento($valor, $pos, $nrDigitoArredonda, self::NR_CINCO);
            }

            // Monta sub-string com o valor acrescido ($sub) completando com zeros ate o numero de digitos.
            $subValor = str_pad($sub, $dig, self::STR_ZERO);

            // Arredonda.
            $valor = $valor + (float)$subValor;

            // Monta sub-string com zero p/ substituicao na parte a direita do dig. de arredondamento (Ex. 0, 00, 000).
            $subStr = str_pad(self::STR_VAZIO, $dig, self::STR_ZERO);

            // Tratamento para arredondamento no primeiro algarismo.
            if ($start === 0) {
                $start++;
                $pos++;
            }

            if ($limiteAux >= $lim) {
                $valor = substr_replace($valor, $subStr, $start, $dig);
            }

            $aux = substr($valor, 0, $pos);
        }

        return (float)$aux;
    }


    /**
     * Calculo do valor que sera acrescido ao digito para o arredondamento.
     *
     * @param float   $valor             Valor a ser manipulado.
     * @param integer $pos               Posicao do caracter delimitador do decimal (virgula).
     * @param integer $nrDigitoArredonda Quantidade de casas decimais para o arredondamento.
     * @param integer $valorBase         Valor Base para subtracao - constante.
     *
     * @return integer
     *
     * @author alessandrobartels - 15/12/2011
     */
    private static function _executaArredondamento($valor, $pos, $nrDigitoArredonda, $valorBase) {

        $dig = abs($nrDigitoArredonda);

        if ($nrDigitoArredonda === 0) {
            $arredondaAux = substr($valor, ($pos-$nrDigitoArredonda-1), 1);
            $valorAux = $valorBase - (int)$arredondaAux;
            $sub = $valorAux;
        } elseif ($nrDigitoArredonda > 0) {
            $arredondaAux = substr($valor, ($pos-$nrDigitoArredonda), 1);
            $valorAux = $valorBase - (int)$arredondaAux;
            $sub = $valorAux;
        } else {
            $arredondaAux = substr($valor, ($pos-$nrDigitoArredonda), 1);
            $valorAux = $valorBase - (int)$arredondaAux;
            $sub = self::STR_ZERO.self::STR_ID_DECIMAL;
            $sub = str_pad($sub, $dig+1, self::STR_ZERO);
            $sub = str_pad($sub, (strlen($sub)+1), $valorAux);
        }

        return $sub;
    }


}
